"""
Models for ResumeManager app.
"""

# Third-Party Imports.
from django.db import models
from django.conf import settings
from django.core.exceptions import ValidationError
from django.utils import timezone
from phonenumber_field.modelfields import PhoneNumberField

# Internal Imports.
from core.models import EmailField, WebsiteField, WorkspaceModel


class Resume(WorkspaceModel):
    """A representation of a resume."""

    # Relationship Keys.
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    theme = models.ForeignKey('ResumeTheme', on_delete=models.CASCADE)

    # Model fields.
    name = models.CharField(max_length=settings.MAX_LENGTH)
    profession = models.CharField(max_length=settings.MAX_LENGTH, blank=True, null=True)
    last_activity = models.DateField(default=timezone.now)

    # Field helper text.
    name.help_text = 'Resume name.'
    last_activity.help_text = 'Last set date of resume updating. Defaults to creation date. Manually set after.'

    class Meta:
        verbose_name = 'Resume'
        verbose_name_plural = 'Resumes'

    def __str__(self):
        return '{0} - {1}'.format(self.name, self.last_activity)


class ResumeTheme(WorkspaceModel):
    """A unique resume theme."""

    # Model fields.
    name = models.CharField(max_length=settings.MAX_LENGTH)
    css_classes = models.CharField(max_length=settings.MAX_LENGTH)
    has_header_image = models.BooleanField(default=False)
    has_footer_image = models.BooleanField(default=False)

    # Field helper text.
    name.help_text = 'Name of theme.'
    has_header_image.help_text = 'Indicates if theme includes header image.'
    has_footer_image.help_text = 'Indicates if theme includes footer image.'

    class Meta:
        verbose_name = 'Resume Theme'
        verbose_name_plural = 'Resume Themes'

    def __str__(self):
        return '{0}'.format(self.name)


class ResumeSection(WorkspaceModel):
    """Formatting of a section/subsection within a resume."""

    # Preset field choices.
    DISPLAY_CATEGORY__NONE = 0
    DISPLAY_CATEGORY__NAME = 1
    DISPLAY_CATEGORY__PROFESSION = 2
    DISPLAY_CATEGORY__CONTACT = 3
    DISPLAY_CATEGORY__EDUCATION = 4
    DISPLAY_CATEGORY__EXPERIENCE = 5
    DISPLAY_CATEGORY__SKILLS = 6
    DISPLAY_CATEGORY_CHOICES = (
        (DISPLAY_CATEGORY__NONE, 'None'),
        (DISPLAY_CATEGORY__CONTACT, 'Name'),
        (DISPLAY_CATEGORY__CONTACT, 'Profession'),
        (DISPLAY_CATEGORY__CONTACT, 'Contact'),
        (DISPLAY_CATEGORY__EDUCATION, 'Education'),
        (DISPLAY_CATEGORY__EXPERIENCE, 'Experience'),
        (DISPLAY_CATEGORY__SKILLS, 'Skills'),
    )

    # Relationship Keys.
    css = models.ManyToManyField('SectionCss', related_name='section')
    resume = models.ForeignKey('Resume', on_delete=models.CASCADE)
    parent_section = models.ForeignKey('self', on_delete=models.CASCADE, blank=True, null=True)

    # Model fields.
    name = models.CharField(max_length=settings.MAX_LENGTH)
    display_category = models.PositiveSmallIntegerField(choices=DISPLAY_CATEGORY_CHOICES, default=0)
    separator_before = models.BooleanField(default=False)
    separator_after = models.BooleanField(default=False)

    # Field helper text.
    css.help_text = 'CSS stylings to apply to section.'
    resume.help_text = 'Resume that section applies to.'
    parent_section.help_text = 'Parent section/subsection to nest within.'
    name.help_text = 'Name to help identify section purpose.'
    display_category.help_text = 'Indicates what content (if any) should display in section.'
    separator_before.help_text = 'Indicates if a separator should be placed before section elements.'
    separator_after.help_text = 'indicates if a separator should be placed after section elements.'

    class Meta:
        verbose_name = 'Resume Section'
        verbose_name_plural = 'Resume Sections'

    def __str__(self):
        return '{0}'.format(self.name)


class SectionCss(WorkspaceModel):
    """Visual style formatting of a section/subsection within a resume."""

    # Model fields.
    name = models.CharField(max_length=settings.MAX_LENGTH)
    css_classes = models.CharField(max_length=settings.MAX_LENGTH)

    # Field helper text.
    name.help_text = 'Name of resume section.'
    css_classes.help_text = 'CSS classes to apply for section.'

    class Meta:
        verbose_name = 'Section CSS'
        verbose_name_plural = 'Section CSS'

    def __str__(self):
        return '{0}'.format(self.name)


class ContactInfo(WorkspaceModel):
    """"""
    # Relationship Keys.
    resume = models.ForeignKey('Resume', on_delete=models.CASCADE)

    # Model fields.
    phone_number = PhoneNumberField(blank=True, null=True)
    email = EmailField(max_length=settings.MAX_LENGTH)
    website = WebsiteField(max_length=settings.MAX_LENGTH, blank=True, null=True)
    git = models.CharField(max_length=settings.MAX_LENGTH, blank=True, null=True)
    linkedin = models.CharField(max_length=settings.MAX_LENGTH, blank=True, null=True)
    art_station = models.CharField(max_length=settings.MAX_LENGTH, blank=True, null=True)

    # Field helper text.
    resume.help_text = 'Resume that contact info applies to. Allows user to have different contact info per resume.'
    website.help_text = 'Personal website, or other professional site not provided here.'
    git.help_text = 'Site to showcase personal git projects.'
    linkedin.help_text = 'LinkedIn site profile.'
    art_station.help_text = 'ArtStation site profile.'

    class Meta:
        verbose_name = 'Contact Info'
        verbose_name_plural = 'Contact Info'

    def __str__(self):
        return '{0} {1} - {2}'.format(self.resume.user.first_name, self.resume.user.last_name, self.resume.name)


class EducationInstance(WorkspaceModel):
    """."""

    # Model fields.
    name = models.CharField(max_length=settings.MAX_LENGTH)
    degree = models.CharField(max_length=settings.MAX_LENGTH)
    date_start = models.DateField()
    date_end = models.DateField(blank=True, null=True)

    # Field helper text.
    name.help_text = 'Name of institution.'
    degree.help_text = 'Degree pursued/acquired.'
    date_start.help_text = 'Starting date of pursuing education.'
    date_end.help_text = 'Ending date of finishing education, or none if ongoing.'

    class Meta:
        verbose_name = 'Education Instance'
        verbose_name_plural = 'Education Instances'

    def __str__(self):
        return '{0} - {1}'.format(self.name, self.degree)


class ExperienceInstance(WorkspaceModel):
    """."""

    # Model fields.
    name = models.CharField(max_length=settings.MAX_LENGTH)
    position = models.CharField(max_length=settings.MAX_LENGTH)
    date_start = models.DateField()
    date_end = models.DateField(blank=True, null=True)

    # Field helper text.
    name.help_text = 'Name of company/workplace.'
    position.help_text = 'Title of position.'
    date_start.help_text = 'Initial hiring date.'
    date_end.help_text = 'Termination date, or none if ongoing.'

    class Meta:
        verbose_name = 'Experience Instance'
        verbose_name_plural = 'Experience Instances'

    def __str__(self):
        return '{0} - {1}'.format(self.name, self.position)


# class ResumeToSectionIntermediary(WorkspaceModel):
#     """An intermediary model that associates a resume and a section.
#
#     Can be nested within other section/subsections, denoted by having a "parent".
#     Each subsection is a unique div.
#     """
#
#     # # Preset field choices.
#     # DISPLAY_CATEGORY__NONE = 0
#     # DISPLAY_CATEGORY__EXPERIENCE = 1
#     # DISPLAY_CATEGORY__EDUCATION = 2
#     # DISPLAY_CATEGORY__SKILLS = 3
#     # DISPLAY_CATEGORY_CHOICES = (
#     #     (DISPLAY_CATEGORY__NONE, 'None'),
#     #     (DISPLAY_CATEGORY__EXPERIENCE, 'Experience'),
#     #     (DISPLAY_CATEGORY__EDUCATION, 'Education'),
#     #     (DISPLAY_CATEGORY__SKILLS, 'Skills'),
#     # )
#
#     # Relationship Keys.
#     resume = models.ForeignKey('Resume', on_delete=models.CASCADE)
#     parent_section = models.ForeignKey('self', on_delete=models.CASCADE, blank=True, null=True)
#     section_format = models.ForeignKey('ResumeSection', on_delete=models.CASCADE)
#     title = models.CharField(max_length=settings.MAX_LENGTH, blank=True, null=True)
#     is_data_subsection = models.BooleanField(default=False)
#     # display_category = models.PositiveSmallIntegerField(choices=DISPLAY_CATEGORY_CHOICES, default=0)
#
#     # Field helper text.
#     resume.help_text = 'The resume to place the section within.'
#     parent_section.help_text = 'Parent section/subsection to nest within.'
#     section_format.help_text = 'The section formatting to use within the resume.'
#     is_data_subsection.help_text = 'Indicates if this is part of a larger parent data section. Results in de-emphasized text.'
#     # display_category.help_text = 'Category of data to display in section.'
#
#     class Meta:
#         verbose_name = 'Resume-to-Section Intermediary'
#         verbose_name_plural = 'Resume-to-Section Intermediaries'
#
#     def __str__(self):
#         return '{0} - {1}'.format(self.resume, self.section_format)
#
#
# class DataSectionHeader(WorkspaceModel):
#     """Implementation of a header for a data section.
#
#     Each data section can only have a single header, but one is not required.
#     """
#
#     # Relationship Keys.
#     resume_section = models.ForeignKey('ResumeToSectionIntermediary', on_delete=models.CASCADE)
#
#     # Model fields.
#     header = models.CharField(max_length=settings.MAX_LENGTH)
#
#     class Meta:
#         verbose_name = 'Data Section Header'
#         verbose_name_plural = 'Data Section Headers'
#
#     def __str__(self):
#         return '{0} - {1}'.format(self.resume_section.resume, self.header[15:])
#
#
# class DataItem(WorkspaceModel):
#     """Implementation of a data item.
#
#     If a data section has only a single item, then the item is displayed in block/paragraph format.
#     If a data section has two or more items, then they are displayed in bullet point format.
#     """
#
#     # Relationship Keys.
#     resume_section = models.ForeignKey('ResumeToSectionIntermediary', on_delete=models.CASCADE)
#
#     # Model fields.
#     title_left = models.CharField(max_length=settings.MAX_LENGTH, blank=True, null=True)
#     title_right = models.CharField(max_length=settings.MAX_LENGTH, blank=True, null=True)
#     item = models.TextField(max_length=500)
#     show_bullet_point = models.BooleanField(default=True)
#     is_emphasized = models.BooleanField(default=False)
#     is_minimized = models.BooleanField(default=False)
#
#     class Meta:
#         verbose_name = 'Data Section Item'
#         verbose_name_plural = 'Data Section Items'
#
#     def __str__(self):
#         return '{0} - {1}'.format(self.resume_section.resume, self.item[15:])
#
#     def clean(self):
#         """
#         Custom cleaning implementation. Includes validation, setting fields, etc.
#         """
#         # Call parent logic.
#         super().clean()
#
#         if self.is_emphasized and self.is_minimized:
#             raise ValidationError('Data cannot be both emphasized and minimized in style.')
#
#
# class DataKeyValue(WorkspaceModel):
#     """Implementation of a data key-value pair."""
#
#     # Relationship Keys.
#     resume_section = models.ForeignKey('ResumeToSectionIntermediary', on_delete=models.CASCADE)
#
#     # Model fields.
#     data_key = models.CharField(max_length=settings.MAX_LENGTH)
#     data_value = models.CharField(max_length=settings.MAX_LENGTH)
#
#     class Meta:
#         verbose_name = 'Data Section Key-Value Pair'
#         verbose_name_plural = 'Data Section Key-Value Pairs'
#
#     def __str__(self):
#         return '{0} - {1}'.format(self.resume_section.resume, self.data_key)
